<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------------------+
// | The Lazym Project                                                  |
// +--------------------------------------------------------------------+
// | Copyright (c) 2004 ູѰ                                      |
// +--------------------------------------------------------------------+
// |  ҽ  Ͽ  LICENSE  LazyBoard ̼ |
// | 1.0   ֽϴ.  ̼    ּҿ  |
// | Ȯΰմϴ.                                                    |
// | http://www.lazybbs.net/license/korean__1_0.txt                     |
// | α׷̳ ̼   gonom9@gmail.com     |
// | Ͻðų http://www.lazybbs.net  湮Ͻñ ٶϴ.            |
// +--------------------------------------------------------------------+
//

/**
 * A Simple xml class
 */
class XML
{
	//{{{ public property
	/**
	 * root element
	 */
	var $root;
	//}}}
	
	//{{{ private property
	
	/**
	 * tag name stack
	 * @var array
	 * @access private
	 */
	var $_tagStack;
	
	/**
	 * xml parser resource
	 * @var resource
	 * @access private
	 */
	var $_parser;
	
	/**
	 * current xml element object
	 * @var object
	 * @access private
	 */
	var $_current;
	
	/**
	 * encoding character
	 * @var string
	 * @access private
	 */
	var $_encoding;
	//}}}
	
	/**
	 * Constructor
	 * @access private
	 */
	function xml()
	{
		if (phpversion() < 5) {
			$this->__construct();
		}
	}
	
	function __construct()
	{
		// Initialize variables
		$this->root = $this->_parser = $this->_current = null;
		$this->_tagStask = array();
	}
	
	function __destruct()
	{
		$this->_parser = null;
	}
	
	function Free()
	{
		if (phpversion() < 5) {
			$this->__destruct();
		}
	}
	
	function &Singleton()
	{
		static $instance;

		if (!isset($instance)) $instance = new XML;
		return $instance;
	}
	
	function &Load($xmlStr)
	{
		$this->root = null;
		$this->_current = null;
		$this->_tagStack = array();
				
		if (is_callable('xml_parse')) {
			$this->_parser = xml_parser_create($this->_encoding);
			
			xml_parser_set_option($this->_parser, XML_OPTION_CASE_FOLDING, false);
			xml_set_object($this->_parser, &$this);
			xml_set_element_handler($this->_parser, '_onOpenTag', '_onCloseTag');
			xml_set_character_data_handler($this->_parser, '_onCData');
			
			xml_parse($this->_parser, $xmlStr);
		} else {
			$offset = 1;
			$xmlStr = preg_replace('/>\s+</', '><', $xmlStr);
			
			while(($pos = strpos($xmlStr, '<', $offset)) !== false) {
				switch ($xmlStr{$pos+1}) {
					case '/': // close tag
						$epos = strpos($xmlStr, '>', $pos);
						$this->_onCloseTag(null, substr($xmlStr, $pos+2, $epos-$pos-2));
						break;
					case '?': // xml declaration
						$epos = strpos($xmlStr, '?>', $pos)+1;
						break;
					default: // open tag
						$epos = strpos($xmlStr, '>', $pos);
						$backstep = $xmlStr{$epos-1}==='/'?0:1;
						$tagStr = substr($xmlStr, $pos+1, $epos-$pos-$backstep);
						list($namespace, $tag, $attributes) = $this->_parseTag($tagStr);
						if ($namespace) $tag = $namespace.':'.$tag;
						$this->_onOpenTag(null, $tag, $attributes);
						
						if ($xmlStr{$epos-1} == '/') {
							$this->_onCloseTag(null, $tag);
						} else if (($pos = strpos($xmlStr, '<', $epos)) !== false && $epos != $pos - 1) {
							$cdata = substr($xmlStr, $epos+1, $pos-$epos-1);
							$cdata = $this->_htmlentity($cdata);
							$this->_onCData(null, $cdata);
						}
						break;
						
				}
				if ($epos !== false) $offset = $epos;
				else $offset = strlen($xmlStr);
			}
		}
	}
	
	function &LoadFile($fileName)
	{
		if (!is_readable($fileName)) {
			Lazym::raiseError('"%s" is NOT readable!', $fileName);
			return null;
		} else {
			$fp = fopen($fileName, 'r');
			$xmlStr = fread($fp, filesize($fileName));
			fclose($fp);
			
			return $this->Load($xmlStr);
		}
	}
	
	function &createRoot($nodeName)
	{
		$this->root = new XMLElement($nodeName);
		return $this->root;
	}
	
	function setEncoding($encoding)
	{
		$this->_encoding = $encoding;
		if ($this->root) $this->root->_encoding = $encoding;
	}
	
	function _onOpenTag($parser, $tag, $attributes)
	{
		$this->_tagStack[] = $tag;
		if (is_null($this->root)) {
			$newElem = &$this->createRoot($tag);
			$newElem->_encoding = $this->_encoding;
		} else {
			$newElem = new XMLElement($tag);
		}
		$newElem->attributes = $attributes;
				
		if ($this->_current) $this->_current->addNode($newElem);
		$this->_current = &$newElem;
	}
	
	function _onCDATA($parser, $cdata)
	{
		if (trim($cdata)) {
			$this->_current->cdata = $cdata;
		}
	}
	
	function _onCloseTag($parser, $tag)
	{
		$lastIndex = count($this->_tagStack) - 1;
		if ($lastIndex < 0) {
			Lazym::raiseError('Invalid index');
		} else if ($this->_tagStack[$lastIndex] !== $tag) {
			Lazym::raiseError('Close tag is not match!');
		} else {
			array_pop($this->_tagStack);
			$this->_current = &$this->_current->parentNode;
		}
	}
	
	function _parseTag($tagStr)
	{
		if ($tagStr !== ltrim($tagStr)) {
			Lazym::raiseError('Tag name cannot have white space!');
		}
		
		$result = array('namespace'=>'','tag'=>'', 'attributes'=>array());
		
		if (($pos=strpos($tagStr, ' ')) === false) $result['tag'] = $tagStr;
		else $result['tag'] = substr($tagStr, 0, $pos);

		if (strpos($result['tag'], ':') !== false) {
			list($result['namespace'], $result['tag']) = explode(':', $result['tag']);
		}
		$pattern = '/([a-z]+)="([^"<>]*)"/U';
		preg_match_all($pattern, trim(substr($tagStr, $pos)), $match, PREG_SET_ORDER);
		$len = count($match);
		for ($i = 0; $i < $len; $i++) {
			$result['attributes'][$match[$i][1]] = $this->_htmlentity($match[$i][2]);
		}
		$result[0] = $result['namespace'];
		$result[1] = $result['tag'];
		$result[2] = $result['attributes'];
	
		return $result;
	}
	
	function _htmlentity($str)
	{
		$entity = array('&nbsp;','&lt;','&gt;','&quot;','&amp;');
		$conv = array(' ','<','>','"','&');
		
		return strtr($str, $entity, $conv);
	}
}

/**
 * Simple XML Element class
 */
class XMLElement
{
	//{{{ public property
	/**
	 * Node name
	 * @var string
	 */
	var $nodeName;
	
	/**
	 * Array of child node
	 * @var array
	 */
	var $attributes;
	
	/**
	 * Array of child node
	 * @var array
	 */
	var $children;
	
	/**
	 * Parent node
	 * @var object
	 */
	var $parentNode;
	
	/**
	 * CData
	 * @var string
	 */
	var $cdata;
	//}}}
	
	//{{{ private property
	//}}}
	
	/**
	 * Constructor
	 */
	function xmlelement($nodeName)
	{
		if (phpversion() < 5) {
			$this->__construct($nodeName);
		}
	}
	
	function __construct($nodeName)
	{
		$this->nodeName = $nodeName;
		$this->children = array();
		$this->attributes = array();
		$this->cdata = null;
	}
	
	function &addNode(&$nodeObject)
	{
		$nodeObject->parentNode = &$this;
		$lastindex = count($this->children);
		$this->children[$lastindex] = $nodeObject;
		
		return $this->children[$lastindex];
	}
	
	function addAttribute($name, $value)
	{
		$this->attribute[$name] = $value;
	}
	
	function xpath($path)
	{
		if ($path{0} == '/') {
			$root = $this->_getRoot();
			$root->xpath(substr($path, 1));
		} else if ($path{0} == '@') {
			
		}
	}
	
	function asXML()
	{
		$result = '';
		$depth = $this->_getDepth();

		if ($depth == 0) {
			$result .= "<?xml version=\"1.0\" encoding=\"".$this->_encoding."\"?>\n";
		}
		$result .= str_repeat("\t", $depth).'<'.$this->nodeName;
		foreach ($this->attributes as $k=>$v) {
			$result .= ' '.$k.'="'.htmlspecialchars($v).'"';
		}
		
		if (count($this->children) > 0) {
			$result .= ">\n";
			foreach($this->children as $child) {
				$result .= $child->asXML();
			}
			$result .= "</".$this->nodeName.">\n";
		} else if (strlen($this->cdata) > 0) {
			$result .= $this->cdata.'</'.$this->nodeName.'>';
		} else {
			$result .= " />\n";
		}
		
		return $result;
	}
	
	function _getDepth()
	{
		if ($this->parentNode) {
			return $this->parentNode->_getDepth() + 1;
		} else {
			return 0;
		}
	}
	
	function &_getRoot()
	{
		if ($this->parentNode) {
			return $this->parentNode->_getRoot();
		} else {
			return $this;
		}
	}
}
?>